/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#define DEBUG_VDCT 0

#define ROUND(x) ((int)((x) + .5))

#include <InterViews/filechooser.h>
#include <InterViews/window.h>
#include <InterViews/display.h>

#include <Unidraw/unidraw.h>
#include <Unidraw/catalog.h>
#include <Unidraw/iterator.h>
#include <Unidraw/editor.h>
#include <Unidraw/statevars.h>
#include <Unidraw/dialogs.h>
#include <Unidraw/viewer.h>
#include <Unidraw/upage.h>

#include <Unidraw/Components/text.h>

#include "dctclasses.h"
#include "dctcat.h"
#include "dcted.h"
#include "dctfile.h"
#include "dctrec.h"
#include "dctcomps.h"
#include "dctext.h"

#include <stdlib.h>
#include <stream.h>
#include <string.h>

extern "C" {
#include "dbStaticLib.h"
};

//------------------------commands for save,saveas,open,new---------------

// Out is a pointer to a string that has not been allocated.  It is
// the caller's duty to free the created string.
static void ConvertString(const char *in, char **out)
{
	if(!in) {
		*out=NULL;
		return;
	}
	int len=strlen(in);
	*out=(char *)calloc(2*len+1,1);
	if(!*out) return;
	const char *pin=in;
	char *pout=*out;
	while(*pin != '\0') {
		switch(*pin) {
		case '\n':
			*pout++='\\';
			*pout='n';
			break;
		case '\t':
			*pout++='\\';
			*pout='t';
			break;
		case '\\':
			*pout++='\\';
			*pout='\\';
			break;
		case '\r':
			*pout++='\\';
			*pout='r';
			break;
		case '\a':
			*pout++='\\';
			*pout='a';
			break;
		case '\b':
			*pout++='\\';
			*pout='b';
			break;
		case '\f':
			*pout++='\\';
			*pout='f';
			break;
		case '\v':
			*pout++='\\';
			*pout='v';
			break;
		default:
			*pout=*pin;
			break;
		}
		pin++;
		pout++;
	}
	*pout='\0';
}

static void UpdateCompNameVars()
{
    Iterator i;
 
    for (unidraw->First(i); !unidraw->Done(i); unidraw->Next(i))
	{
        Editor* ed = unidraw->GetEditor(i);
        CompNameVar* compNameVar = (CompNameVar*) ed->GetState("CompNameVar");
        if (compNameVar != nil) compNameVar->UpdateName();
    }
}

static boolean Writable (Component* comp)
{
	Catalog* catalog = unidraw->GetCatalog();
	const char* name = catalog->GetName(comp);
	return name == nil || (catalog->Exists(name) && catalog->Writable(name));
}

static boolean ReadyToClose (Editor* ed)
{
    ModifStatusVar* mv = (ModifStatusVar*) ed->GetState("ModifStatusVar");
 
    if (mv != nil && Writable(mv->GetComponent()) && mv->GetModifStatus())
	{
        ConfirmDialog dialog("Save changes?");
 
        ed->InsertDialog(&dialog);
        char resp = dialog.Confirm();
        ed->RemoveDialog(&dialog);
 
        if (resp == '\007')
            return false;                       // confirm dialog aborted
		else if (resp == 'y')
		{
            DctSaveCompCmd saveComp(ed);
            saveComp.Execute();
 
            if (mv->GetModifStatus())
                return false;                   // save dialog was aborted
        }
    }
    return true;
}

//---new component cmd---

DctNewCompCmd::DctNewCompCmd(ControlInfo* c, Component* p):Command(c)
{ prototype_=p; }
DctNewCompCmd::DctNewCompCmd(Editor* e, Component* p):Command(e)
{ prototype_=p; }

DctNewCompCmd::~DctNewCompCmd () { delete prototype_; }
boolean DctNewCompCmd::Reversible () { return false; }
ClassId DctNewCompCmd::GetClassId() { return DCTNEWCOMP_CMD; }
boolean DctNewCompCmd::IsA(ClassId id)
{ return DCTNEWCOMP_CMD == id || Command::IsA(id); }

void DctNewCompCmd::Execute()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	Dct* ed = (Dct*)GetEditor();
	Component* orig = ed->GetComponent();
	Component* comp = prototype_->Copy();
	NameVar* nv = (NameVar*)ed->GetState("NameVar");
	CompNameVar* compNameVar = (CompNameVar*) ed->GetState("CompNameVar");
    ModifStatusVar* modifVar = (ModifStatusVar*)ed->GetState("ModifStatusVar");

	if(!ReadyToClose(ed)) return;

    if (compNameVar != nil) compNameVar->SetComponent(comp);
    if (modifVar != nil) modifVar->SetComponent(comp);
 
    ed->SetComponent(comp);
    ed->Update();
 
    if (orig != nil && unidraw->FindAny(orig) == nil)
	{
        Component* root = orig->GetRoot();
        delete root;
    }

	dbFreeRecords(db);
	nv->SetName(du->GetName());
}

Command* DctNewCompCmd::Copy()
{
	Command* copy = new DctNewCompCmd(CopyControlInfo(), prototype_->Copy());
	InitCopy(copy);
	return copy;
}

void DctNewCompCmd::Read(istream& in)
{ Command::Read(in); prototype_ = unidraw->GetCatalog()->ReadComponent(in); }
void DctNewCompCmd::Write(ostream& out)
{ Command::Write(out); unidraw->GetCatalog()->WriteComponent(prototype_, out); }

//---save component cmd---
 
boolean DctSaveCompCmd::IsA(ClassId id)
{ return SAVECOMP_CMD == id || Command::IsA(id); }
ClassId DctSaveCompCmd::GetClassId () { return SAVECOMP_CMD; }
boolean DctSaveCompCmd::Reversible () { return false; }
DctSaveCompCmd::DctSaveCompCmd (ControlInfo* c) : Command(c) { }
DctSaveCompCmd::DctSaveCompCmd (Editor* ed) : Command(ed) { }
 
Command* DctSaveCompCmd::Copy()
{
    Command* copy = new DctSaveCompCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void DctSaveCompCmd::Execute()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	Dct* ed = (Dct*)GetEditor();
	NameVar* nv = (NameVar*)ed->GetState("NameVar");
	ModifStatusVar* modifVar = (ModifStatusVar*)ed->GetState("ModifStatusVar");
    CompNameVar* compNameVar = (CompNameVar*) ed->GetState("CompNameVar");
    const char* name = (compNameVar == nil) ? nil : compNameVar->GetName();

	if (name == nil)
	{
        DctSaveCompAsCmd saveCompAs(ed);
        saveCompAs.Execute();
	}
	else if(modifVar == nil || modifVar->GetModifStatus())
	{
		DctCatalog* catalog = (DctCatalog*)unidraw->GetCatalog();
		Component* comp;

		if((catalog->Retrieve(name, comp) && catalog->Save(comp,name,"scum")))
		{
			if (modifVar != nil) modifVar->SetModifStatus(false);
			unidraw->ClearHistory(comp);
		}
		else
		{
            // char title[CHARBUFSIZE];
            // const char* reason=!Writable(comp) ? "(File not writable.)" : "";
            // sprintf(title, "Couldn't save! %s", reason);
            // char subtitle[CHARBUFSIZE];
            // const char* domain=unidraw->GetCatalog()->GetAttribute("domain");
            // domain = (domain == nil) ? "component" : domain;
            // sprintf(subtitle, "Save this %s as:", domain);
 
            DctSaveCompAsCmd saveCompAs(ed);
            saveCompAs.Execute();
		}
	}
}

//---save as component cmd---

ClassId DctSaveCompAsCmd::GetClassId() { return DCTSAVECOMPAS_CMD; }
boolean DctSaveCompAsCmd::IsA(ClassId id)
 { return DCTSAVECOMPAS_CMD == id || Command::IsA(id); }
 
DctSaveCompAsCmd::DctSaveCompAsCmd(ControlInfo* c):Command(c) { }
DctSaveCompAsCmd::DctSaveCompAsCmd(Editor* ed):Command(ed) { }
DctSaveCompAsCmd::~DctSaveCompAsCmd () { }
 
Command* DctSaveCompAsCmd::Copy()
{
    Command* copy = new DctSaveCompAsCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

boolean DctSaveCompAsCmd::Reversible () { return false; }

void DctSaveCompAsCmd::Execute()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	Dct* ed = (Dct*)GetEditor();
	NameVar* nv = (NameVar*)ed->GetState("NameVar");
	ModifStatusVar* modifVar = (ModifStatusVar*)ed->GetState("ModifStatusVar");
	DctCatalog* catalog = (DctCatalog*)unidraw->GetCatalog();
	boolean complete = false;
	char buf[100];
	char *name,*db_name,*p;
	CompNameVar* cnv;
	Component* comp;
	FILE* fp;

	// FileChooser fc("file_chew","Save As","",".",10,24,"Save",Center);
	FileDialog fc("File Chew","Save As");

    ed->InsertDialog(&fc);
	global_dialog_wn_delete(&fc);

	while(complete==false)
	{
		switch(fc.GetSaveName(name))
		{
		case FDIAL_CANCEL: complete=true; break;
		case FDIAL_INVALID:
			{
				AcknowledgeDialog ack("Invalid name!",
				  "Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			fc.Update();
			break;
		case FDIAL_DBD:
			if((fp=fopen(name,"w")))
			{
				dbWriteMenuFP(db,fp,NULL);
				dbWriteRecordTypeFP(db,fp,NULL);
				dbWriteDeviceFP(db,fp);
				dbWriteBreaktableFP(db,fp);
				fclose(fp);
			}
			else
			{
				AcknowledgeDialog ack("Could not write in the dbd file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			complete=true;
			break;
		case FDIAL_DB:
			if(dbWriteRecord(db,name,NULL,1))
			{
				AcknowledgeDialog ack("Could not write in the db file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			complete=true;
			break;
		case FDIAL_NORMAL:
			if(catalog->Exists(name) && catalog->Writable(name))
			{
				// file exists already
				ConfirmDialog dialog(name,"File exists, overwrite it?");
				ed->InsertDialog(&dialog);
				char confirmation = dialog.Confirm();
				ed->RemoveDialog(&dialog);
				if (confirmation != 'y') complete=true;
			}
			if(complete==false)
			{
				cnv=(CompNameVar*)ed->GetState("CompNameVar");
				const char* oldname=(cnv==nil)?nil:cnv->GetName();
				comp=ed->GetComponent();

				if(catalog->Exists(name) && !catalog->Writable(name))
				{
#if 1
					AcknowledgeDialog ack("File selected is not writable!",
					  "Error");
					ed->InsertDialog(&ack);
					ack.Acknowledge();
					ed->RemoveDialog(&ack);
#else
					// KE: This doesn't do anything
					fc.SetSubtitle("File selected is not writable!");
#endif
					fc.Update();
				}
				else
				{
					if(oldname==nil)
						comp=comp->GetRoot();
					else
					{
						catalog->Retrieve(oldname,comp);
						catalog->Forget(comp);
					}
	
					modifVar=
						(ModifStatusVar*)ed->GetState("ModifStatusVar");

					if(catalog->Save(comp,name,"scum"))
					{
						if(modifVar!=nil)
							modifVar->SetModifStatus(false);

						unidraw->ClearHistory(comp);
						UpdateCompNameVars();
						du->SetName(name);
						nv->SetName(name);
						complete=true;
					}
					else
					{
						if(modifVar!=nil) modifVar->Notify();
						UpdateCompNameVars();
#if 1
						AcknowledgeDialog ack("File could not be saved!",
						  "Error");
						ed->InsertDialog(&ack);
						ack.Acknowledge();
						ed->RemoveDialog(&ack);
#else
						// KE: This doesn't do anything
						fc.SetSubtitle("File could not be saved!");
#endif
						fc.Update();
					}
				}
			}
			break;
		default: break; // ??????????????
		}
		if(name) delete [] name;
	}

	ed->RemoveDialog(&fc);
}

//---save as vdct component cmd---

ClassId DctSaveCompAsVdctCmd::GetClassId() { return DCTSAVECOMPASVDCT_CMD; }
boolean DctSaveCompAsVdctCmd::IsA(ClassId id)
{ return DCTSAVECOMPASVDCT_CMD == id || Command::IsA(id); }

DctSaveCompAsVdctCmd::DctSaveCompAsVdctCmd(ControlInfo* c):Command(c) { }
DctSaveCompAsVdctCmd::DctSaveCompAsVdctCmd(Editor* ed):Command(ed) { }
DctSaveCompAsVdctCmd::~DctSaveCompAsVdctCmd () { }

Command* DctSaveCompAsVdctCmd::Copy()
{
    Command* copy = new DctSaveCompAsVdctCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

boolean DctSaveCompAsVdctCmd::Reversible () { return false; }

void DctSaveCompAsVdctCmd::Execute()
{
	DctUnidraw *du = (DctUnidraw*)unidraw;
	DBBASE *db = du->GetDatabase();
	DBENTRY *dbe;
	Dct *ed = (Dct *)GetEditor();
	NameVar *nv = (NameVar*)ed->GetState("NameVar");
	ModifStatusVar *modifVar = (ModifStatusVar*)ed->GetState("ModifStatusVar");
	DctCatalog *catalog = (DctCatalog*)unidraw->GetCatalog();
	boolean complete = false, compok = true, errors=false;;
	char buf[100];
	char *name,*db_name,*p;
	CompNameVar *cnv;
	Component *comp;
	FILE *fp=NULL;
	Iterator it;				
	long rc;
	int textid=0,ok=1;
	int lvdct=0,bvdct=0,rvdct=0,tvdct=0;
	
#if DEBUG_VDCT
	printf("DctSaveCompAsVdctCmd::Execute\n");

#if DEBUG_VDCT
	// Coordinate trial and error
	{
		ManagedWindow* mw = ed->GetWindow();
		Canvas *c=mw->canvas();
		Display *d=mw->display();
		printf("  Canvas: %g x %g [%d x %d]\n",
		  c->width(),c->height(),c->pwidth(),c->pheight());
		printf("  Display: %g x %g [%d x %d]\n",
		  d->width(),d->height(),d->pwidth(),d->pheight());
		
		Viewer *v=ed->GetViewer();
		UPage *up=v->GetPage();
		Graphic *pg=up->GetGraphic();
		Graphic *g=v->GetGraphic();
		// Coord here is apparently an int
		// Width is right-left+1, different from Window coordinates
		Coord xleft,xbottom,xright,xtop;
		float bleft,bbottom,bright,btop,x,y;
		
#if 0
		// Viewer::GetGraphicBox is protected
		v->GetGraphicBox(xleft,xbottom,xright,xtop);
		printf(" Viewer:\n");
		printf("  Box: top=%g bottom=%g right=%g top=%g\n"
		  "          width=%g height=%g\n",
		  (float)xleft,(float)xbottom,(float)xright,(float)xtop,
		  (float)(xright-xleft+1),(float)(xtop-xbottom+1));
#endif
		
		g->GetBox(xleft,xbottom,xright,xtop);
		g->GetBounds(bleft,bbottom,bright,btop);
		g->GetCenter(x,y);
		printf(" Viewer Graphic:\n");
		printf("  Box: top=%g bottom=%g right=%g top=%g\n"
		  "          width=%g height=%g\n",
		  (float)xleft,(float)xbottom,(float)xright,(float)xtop,
		  (float)(xright-xleft+1),(float)(xtop-xbottom+1));
		printf("  Bounds: left=%g bottom=%g right=%g top=%g\n"
		  "          width=%g height=%g\n",
		  bleft,bbottom,bright,btop,bright-bleft+1,btop-bbottom+1);
		printf("  Center: x=%g y=%g\n",x,y);
		
		pg->GetBox(xleft,xbottom,xright,xtop);
		pg->GetBounds(bleft,bbottom,bright,btop);
		pg->GetCenter(x,y);
		printf(" UPage Graphic:\n");
		printf("  Box: top=%g bottom=%g right=%g top=%g\n"
		  "          width=%g height=%g\n",
		  (float)xleft,(float)xbottom,(float)xright,(float)xtop,
		  (float)(xright-xleft+1),(float)(xtop-xbottom+1));
		printf("  Bounds: left=%g bottom=%g right=%g top=%g\n"
		  "          width=%g height=%g\n",
		  bleft,bbottom,bright,btop,bright-bleft+1,btop-bbottom+1);
		printf("  Center: x=%g y=%g\n",x,y);
	}
#endif
#endif
	// FileChooser fc("file_chew","Save As","",".",10,24,"Save",Center);
	FileDialog fc("File Chew","Save As VDCT");
	fc.SetTextFilter(".*\\.vdb*$");
	fc.Update();
	
    ed->InsertDialog(&fc);
	global_dialog_wn_delete(&fc);
	
	while(complete==false)
	{
		switch(fc.GetSaveName(name))
		{
		case FDIAL_CANCEL:
#if DEBUG_VDCT
			printf("  FDIAL_CANCEL\n");
#endif
			complete=true;
			break;
		case FDIAL_INVALID:
		case FDIAL_NORMAL:
		case FDIAL_DB:
		case FDIAL_DBD:
#if DEBUG_VDCT
			printf("  FDIAL_INVALID, FDIAL_NORMAL, FDIAL_DBD, or FDIAL_NORMAL\n");
#endif
			{
				AcknowledgeDialog ack("File must be .vdb",
				  "Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			fc.Update();
			break;
		case FDIAL_VDB:
			printf("Save as vdb file: %s\n",name);
#if DEBUG_VDCT
			printf("  FDIAL_VDB\n");
#endif
			// Check if it exists
			if(catalog->Exists(name) && catalog->Writable(name))
			{
				ConfirmDialog dialog(name,"File exists, overwrite it?");
				ed->InsertDialog(&dialog);
				char confirmation = dialog.Confirm();
				ed->RemoveDialog(&dialog);
				if (confirmation != 'y') {
					complete=true;
					break;
				}
			}
			// Open the file
			if(!(fp=fopen(name,"w")))
			{
				fprintf(fp,"\n\n## Could not open .vdb file\n\n");
				AcknowledgeDialog ack("Could not open .vbd file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
				errors=true;
				complete=true;
				break;
			}
			// Write the header
			fprintf(fp,"#! Generated by VisualDCT v2.4\n");
			fprintf(fp,"#! DBDSTART\n");
			const Uninode *node;
			for(node=du->GetDBDList(); node; node=node->next) {
				fprintf(fp,"#! DBD(\"%s\")\n",node->string);
			}
			fprintf(fp,"#! DBDEND\n");
			fprintf(fp,"\n\n");
			// Write the database part (why we use FILE instead of
			// ostream)
			if(dbWriteRecordFP(db,fp,NULL,1))
			{
				fprintf(fp,"\n\n## Could not write db part of vdb file\n\n");
				AcknowledgeDialog ack("Could not write db part of vdb file",
				  "Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
				fclose(fp);
				errors=true;
				complete=true;
				break;
			}
			// Write the graphical info header
			fprintf(fp,"\n");
			fprintf(fp,"#! Further lines contain data used by VisualDCT\n");
			fprintf(fp,"#! View(0,0,0.2)\n");
			
			// Get the DCTGRAPHIC_COMPS component of the editor.  comp
			// should be a DCTGRAPHIC_COMPS, hence a GRAPHIC_COMPS.
			compok=true;
			comp=ed->GetComponent();
			if(!comp || !comp->IsA(GRAPHIC_COMPS)) {
				char string[512];
				sprintf(string,"Bad component (%lu). "
				  "Conversion results will vary.",
				  comp->GetClassId());
				fprintf(fp,"\n\n## %s\n\n",string);
				AcknowledgeDialog ack(string,"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
				errors=true;
				compok=false;
				break;
			}
			// Loop over records.  The primary reason for doing this
			// here is that VDCT directives apparently need to be in
			// database order.
			dbe=dbAllocEntry(db);
			// Loop over fields
			rc=dbFirstRecordType(dbe);
			while(!rc) {
				// Loop over records
				rc=dbFirstRecord(dbe);
				while(!rc) {
					if(!dbIsVisibleRecord(dbe)) {
						// Put them in the same location at bottom right
						int dctonly=TRUE;
						lvdct=VDCT_VIRTUAL_WIDTH-240;
						tvdct=VDCT_VIRTUAL_HEIGHT-300;
						fprintf(fp,"#! Record(\"%s\",%d,%d,0,0,\"%s\")\n",
						  dbGetRecordName(dbe),lvdct,tvdct,dbGetRecordName(dbe));
						// Loop over fields and make them invisible
						rc=dbFirstField(dbe,dctonly);
						while(!rc) {
							// 0=NON_DEFAULT_VISIBLE
							// 1=ALWAYS_VISIBLE 
							// 2=NEVER_VISIBLE 
							int visibility=2;
							fprintf(fp,"#! Visibility(\"%s.%s\",%d)\n",
							  dbGetRecordName(dbe),dbGetFieldName(dbe),
							  visibility);
							rc=dbNextField(dbe,dctonly);
						}
					} else if(compok) {
						// Search for the RecComp with this name
						for (comp->First(it); !comp->Done(it);
							 comp->Next(it)) {
							GraphicComps *gcomps=(GraphicComps *)comp;
							GraphicComp *gcomp=gcomps->GetComp(it);
							ClassId id=gcomp->GetClassId();  // ulong
							if(id != REC_COMP) continue;
							RecComp *rcomp=(RecComp *)gcomp;
							DBENTRY *rec=rcomp->GetRec();
							if(rec && !strcmp(dbGetRecordName(dbe),
								 dbGetRecordName(rec))) {
								// Write the record
								rcomp->WriteVDCT(fp);
								ok=true;
								break;
							}
						}
					} else {
						// Do nothing and let VDCT handle it
					}
					rc=dbNextRecord(dbe);
				}
				rc=dbNextRecordType(dbe);
			}
			dbFreeEntry(dbe);

			// Loop over the components to pick up TEXT_COMP
			if(comp) {
				for (comp->First(it); !comp->Done(it); comp->Next(it)) {
					GraphicComps *gcomps=(GraphicComps *)comp;
					GraphicComp *gcomp=gcomps->GetComp(it);
					ClassId id=gcomp->GetClassId();  // ulong
#if DEBUG_VDCT
					printf("Child %2d %lu\n",i,gcomp->GetClassId());
#endif
					switch(id) {
					case TEXT_COMP:
					{
						textid++;
						TextComp *tcomp=(TextComp *)gcomp;
						TextGraphic *tg=tcomp->GetText();
						float left=0,bottom=0,right=0,top=0;
						tg->GetBounds(left,bottom,right,top);
						char *string=NULL;
						fprintf(fp,"#! TextBox(TB%d,",textid);
						PageToVDCT(left,bottom,lvdct,bvdct);
						PageToVDCT(right,top,rvdct,tvdct);
						fprintf(fp,"%d,%d,%d,%d,",lvdct,bvdct,rvdct,tvdct);
						fprintf(fp,"0,\"Helvetica\",14,0,16777215,");
						ConvertString(tg->GetOriginal(),&string);
						fprintf(fp,"\"%s\"\n",string?string:"");
						if(string) free(string);
						break;
					}
					case REC_COMP:
					case NODE_COMP:
					case WIRE_COMP:
						// Handled above in looping over records.
						// NODE_COMP and WIRE_COMP are handled in
						// REC_COMP (if at all).
						break;
					case EXTLINK_COMP:
					{
						ExtLinkComp *ecomp=(ExtLinkComp *)gcomp;
						// This is not handled, so it is an error
						ecomp->WriteVDCT(fp);
						errors=true;
						break;
					}
					default:
						fprintf(fp,
						  "\n\n## Unhandled GDCT component type [%lu]\n\n",
						  id);
						errors=true;
						break;
					}
				}
#if DEBUG_VDCT
				printf("Component %u nChildren=%d\n",comp->GetClassId(),i);
				printf("End debug\n");
#endif
			}
			// Close the file
			fclose(fp);
			complete=true;
			break;
		default:
#if DEBUG_VDCT
			printf("  default\n");
#endif
			break; // ??????????????
		}
		if(name) delete [] name;
	}
	
	ed->RemoveDialog(&fc);
// Display dialog if there were errors
	if(errors) {
		AcknowledgeDialog ack("There were errors converting to VDCT. "
		  "Look in the .vdb file for lines with ##.",
		  "Error");
		ed->InsertDialog(&ack);
		ack.Acknowledge();
		ed->RemoveDialog(&ack);
	}
#if DEBUG_VDCT
	printf("End DctSaveCompAsVdctCmd::Execute\n");
#endif
}

//---view component cmd---

DctViewCompCmd::DctViewCompCmd(ControlInfo* c) : Command(c) { }
DctViewCompCmd::DctViewCompCmd(Editor* ed) : Command(ed){ }
DctViewCompCmd::~DctViewCompCmd () { }
boolean DctViewCompCmd::Reversible () { return false; }
ClassId DctViewCompCmd::GetClassId() { return DCTVIEWCOMP_CMD; }
boolean DctViewCompCmd::IsA(ClassId id)
{ return DCTVIEWCOMP_CMD == id || Command::IsA(id); }

Command* DctViewCompCmd::Copy ()
{
    Command* copy = new DctViewCompCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void DctViewCompCmd::Execute()
{
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetDatabase();
	Dct* ed = (Dct*)GetEditor();
	NameVar* nv = (NameVar*)ed->GetState("NameVar");
	ModifStatusVar* modif = (ModifStatusVar*) ed->GetState("ModifStatusVar");
	CompNameVar* cnv = (CompNameVar*)ed->GetState("CompNameVar");

	if( !ReadyToClose(ed) ) return;

	boolean complete = false;
	// FileChooser fc("file_chew","Open","",".",10,24,"Open",Center);
	FileDialog fc("File Chew","Open");
	DctCatalog* catalog = (DctCatalog*)du->GetCatalog();
	GraphicComp* comp;
	char* name;

	ed->InsertDialog(&fc);
	global_dialog_wn_delete(&fc);

	while(complete==false)
	{
		switch(fc.GetOpenName(name))
		{
		case FDIAL_CANCEL:
			complete=true;
			break;
		case FDIAL_INVALID:
			{
				AcknowledgeDialog ack("Invalid name!",
				  "Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			fc.Update();
			break;
		case FDIAL_DBD:
			if(du->AddDBD(name))
			{
				AcknowledgeDialog ack("Could not read in the dbd file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			else
			{
				if(modif != nil) modif->SetModifStatus(true);
			}
			complete=true;
			break;
		case FDIAL_DB:
			if(dbReadDatabase(&db,name,NULL,NULL))
			{
				AcknowledgeDialog ack("Could not read in the db file",
					"Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
			}
			else
			{
				if(modif != nil) modif->SetModifStatus(true);
			}
			complete=true;
			break;
		case FDIAL_NORMAL:
			// the user hit ok button
			du->MakeCatDatabase();
			
			if(catalog->Retrieve("you_scum",name, (Component*&) comp))
			{
				Component* orig = ed->GetComponent();
				ed->SetComponent(comp);
				unidraw->Update();

				if(cnv != nil) cnv->SetComponent(comp);
				if(modif != nil) modif->SetComponent(comp);

				if(orig != nil && unidraw->FindAny(orig) == nil)
				{
					Component* root = orig->GetRoot();
					delete root;
				}

				du->UseCatDatabase();
				DBBASE* dbx = du->GetDatabase();
				ed->SetContainer(dbx);
				nv->SetName(du->GetName());
				complete=true;
			}
			else
			{
#if 1
				AcknowledgeDialog ack("Could not read the file!",
				  "Error");
				ed->InsertDialog(&ack);
				ack.Acknowledge();
				ed->RemoveDialog(&ack);
#else
				// KE: This doesn't do anything
				fc.SetSubtitle("Could not read the file!");
#endif
				fc.Update();
			}
			break;
		default: break; // ?????????????
		}
		if(name) delete [] name;
	}
	ed->RemoveDialog(&fc);
}

/* **************************** Emacs Editing Sequences ***************** */
/* Local Variables: */
/* tab-width: 4 */
/* c-basic-offset: 4 */
/* c-comment-only-line-offset: 0 */
/* c-file-offsets: ((substatement-open . 0) (label . 0)) */
/* End: */
