/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef dctcomps_h
#define dctcomps_h

#include "dctvars.h"

#include <InterViews/rubverts.h>
#include <Unidraw/iterator.h>
#include <Unidraw/Components/pin.h>
#include <Unidraw/Graphic/picture.h>
#include <Unidraw/Components/text.h>
#include <Unidraw/Components/psview.h>
#include <Unidraw/Components/grcomp.h>
#include <Unidraw/Components/grview.h>

class MultiLine;
class NodeVar;
class PVLinkVar;
class CGlue;
class Connector;
class TextGraphic;

extern "C" {
#include "dbStaticLib.h"
};

//------------------------pp/ms graphic-----------------------------
class PropGraphic: public Picture
{
public:
	PropGraphic(const char*);
	virtual ~PropGraphic();

	const char* GetString();
private:
	TextGraphic* proc_name;
};

inline const char* PropGraphic::GetString() { return proc_name->GetOriginal(); }

// --------------------------rubber wire stuff----------------------

class RubberWire : public GrowingVertices {
public:
    RubberWire(Painter*, Canvas*, Coord[], Coord[], int, boolean =
true);
 
    virtual void AppendVertex(Coord, Coord);
    virtual void Draw();
 
    void Manhattan(boolean);
private:
    void ConstrainVertices(Coord, Coord);
    void Orthonormalize(Coord, Coord);
private:
    boolean _manhattan;
};

// --------------------------node component stuff----------------------

typedef enum { ConNode, ExtNode, FldNode } NodeType;

class NodeComp : public PinComp
{
public:
    NodeComp(PinGraphic* = nil);

    NodeVar* GetNodeVar();
	DBENTRY* GetRecord();
	DBENTRY* GetField();
	void SetRecord(DBENTRY*);
	void SetSide(char);
	char GetSide();
	char* GetFieldName();
	char* GetRecordName();
	boolean HasConnections();
	NodeType GetNodeType();
	void SetNodeType(NodeType);
	NodeComp* FindNodeToConnect();

	void Read(istream&);
	void Write(ostream&);
	void WriteVDCT(FILE *fp);

	virtual void Connect(Connector*,CGlue* = nil);
	virtual void Disconnect(Connector*);
	virtual void Interpret(Command*);

	virtual void Update();
    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    friend class NLNode;
    UList* GetConnections();
	DBENTRY _record;
	char _side;
	NodeType _node_type;
	int _record_valid,_field_valid;
};

inline void NodeComp::SetNodeType(NodeType nt) { _node_type = nt; }
inline NodeType NodeComp::GetNodeType() { return _node_type; }
inline UList* NodeComp::GetConnections () { return _cnxns; }
inline char NodeComp::GetSide() { return _side; }

inline DBENTRY* NodeComp::GetField()
	{ return (_record_valid&&_field_valid)?&_record:NULL; }
inline DBENTRY* NodeComp::GetRecord()
	{ return (_record_valid&&_field_valid)?&_record:NULL; }
inline void NodeComp::SetRecord(DBENTRY* r)
	{ dbCopyEntryContents(r,&_record); _record_valid=1; _field_valid=1; }

// --------------------------node view stuff----------------------

class NodeView : public PinView
{
public:
    NodeView(NodeComp* = nil);

	NodeComp* GetNodeComp();
	virtual void Update();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
    virtual PinComp* NewSubject(PinGraphic*);
};

// --------------------------wire component stuff----------------------

class WireComp : public GraphicComp
{
public:
    WireComp();
    WireComp(Coord* x, Coord* y, int count, Graphic*);
    virtual ~WireComp();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual void Update();
    virtual void First(Iterator&);
    virtual void Last(Iterator&);
    virtual void Next(Iterator&);
    virtual void Prev(Iterator&);
    virtual boolean Done(Iterator);

    virtual GraphicComp* GetComp(Iterator);
    virtual void SetComp(GraphicComp*, Iterator&);
    virtual void SetMobility(Mobility);
    virtual TransferFunct* GetTransferFunct();

    void GetNodes(NodeComp*&, NodeComp*&);
    MultiLine* GetMultiLine();

	PVLinkVar* GetPVLinkVar();
	void SetPVLinkVar(PVLinkVar*);

	void AlignNode();

    virtual Component* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    void WriteVDCT(FILE *fp);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    NodeComp *_node1, *_node2;
    class TF_Direct* _transfn;

	// added for use in dct
	PVLinkVar* _pvlink;
	PropGraphic* _prop_graphic;
};

inline void WireComp::SetPVLinkVar(PVLinkVar* pv) { _pvlink=pv; }
inline PVLinkVar* WireComp::GetPVLinkVar() { return _pvlink; }

// --------------------------wire view stuff----------------------

class WireView : public GraphicView {
public:
    WireView(WireComp* = nil);
    virtual ~WireView();

    virtual void Update();
    virtual Manipulator* CreateManipulator(Viewer*,Event&,Transformer*,Tool*);
    virtual Command* InterpretManipulator(Manipulator*);

    virtual void First(Iterator&);
    virtual void Last(Iterator&);
    virtual void Next(Iterator&);
    virtual void Prev(Iterator&);
    virtual boolean Done(Iterator);

    virtual GraphicView* GetView(Iterator);
    virtual void SetView(GraphicView*, Iterator&);

    WireComp* GetWireComp();
    virtual Graphic* GetGraphic();

	void AlignNode();

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
    virtual void CreateHandles();
private:
    class MultiLine* GetMultiLine();
    void GetEndpoints(Coord&, Coord&, Coord&, Coord&);
    void GetVertices(Coord*&, Coord*&, int&);
    void UpdateMultiLine();

    Manipulator* CreateWireCompManip(Viewer*, Event&, Transformer*, Tool*);
    Command* InterpWireCompManip(Manipulator*);
private:
    NodeView *_nodeView1, *_nodeView2;
	PropGraphic* _prop_graphic;
};

#endif
