/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include <InterViews/menu.h>
#include <InterViews/rubverts.h>
#include <InterViews/rubline.h>
#include <Unidraw/viewer.h>
#include <Unidraw/selection.h>
  
#include "dctclasses.h"
#include "dctmanip.h"
#include "dctcomps.h"
#include "dctcmds.h"

#include <stream.h>
#include <string.h>

//-------------------popup item stuff-------------------------
class PopupItem : public MenuItem
{
public:
    PopupItem(const char*, Alignment, Command*);
    virtual void Do();
private:
    Command* _cmd;
};
 
PopupItem::PopupItem ( const char* str, Alignment al, Command* cmd) 
	: MenuItem(str, al)
{
    _cmd = cmd;
}
 
void PopupItem::Do () { _cmd->Execute(); }

//-------------------examine tool stuff----------------------------
ExamineTool::ExamineTool (ControlInfo* m) : Tool(m)
{
	field_edit_cmd = new FieldEditCmd;
	field_export_cmd = new FieldExportCmd;
	rename_rec_cmd = new RenameRecCmd;
	// cont_cmd = new ContCmd;
}

ExamineTool::~ExamineTool ()
{
	delete field_edit_cmd;
	delete field_export_cmd;
	delete rename_rec_cmd;
	// delete cont_cmd;
}

Tool* ExamineTool::Copy () { return new ExamineTool(CopyControlInfo()); }

ClassId ExamineTool::GetClassId () { return EXAMINE_TOOL; }
boolean ExamineTool::IsA (ClassId id) {return EXAMINE_TOOL==id||Tool::IsA(id);}

Manipulator* ExamineTool::CreateManipulator (Viewer* v, Event& e,Transformer*)
{
    GraphicView* views = v->GetGraphicView();
    Selection* s = v->GetSelection(), *newSel;
    Manipulator* m = nil;
 
    s->Clear();
    newSel = views->ViewIntersecting(e.x-SLOP, e.y-SLOP, e.x+SLOP, e.y+SLOP);
 
    if (!newSel->IsEmpty())
	{
        Iterator i;
        newSel->First(i);
		GraphicView* gv = newSel->GetView(i);
 
        s->Append(gv);
        s->Update();
 
        InitCmds(v, gv->GetGraphicComp());
        m = CreatePopupManip(v,gv->GetGraphicComp());
    }
    delete newSel;
    return m;
}

void ExamineTool::InitCmds(Viewer* v, GraphicComp* comp)
{
    Editor* ed = v->GetEditor();
 
    field_export_cmd->SetGraphicComp(comp);
    field_edit_cmd->SetGraphicComp(comp);
    rename_rec_cmd->SetGraphicComp(comp);
    // cont_cmd->SetGraphicComp(comp);
 
    field_export_cmd->SetEditor(ed);
    field_edit_cmd->SetEditor(ed);
    rename_rec_cmd->SetEditor(ed);
    // cont_cmd->SetEditor(ed);
}
 
Manipulator* ExamineTool::CreatePopupManip (Viewer* v,GraphicComp* gc) {
    PopupMenu* popup = new PopupMenu;
 
	if(gc->IsA(REC_COMP))
    	popup->Include(
			new PopupItem("Field Export List",Center,field_export_cmd));

	if(!gc->IsA(EXTLINK_COMP))
    	popup->Include(new PopupItem("Edit Attributes",Center,field_edit_cmd));

	if(gc->IsA(REC_COMP))
    	popup->Include(new PopupItem("Rename Record",Center,rename_rec_cmd));
 
	if(gc->IsA(EXTLINK_COMP))
    	popup->Include(new PopupItem("Rename External Link",Center,rename_rec_cmd));

    return new PopupManip(v, popup, this);
}

//------------------------popup manipulator---------------------------
PopupManip::PopupManip(Viewer* viewer, PopupMenu* popup, Tool* tool)
{
    _viewer = viewer;
    _popup = popup;
    _tool = tool;
}
 
PopupManip::~PopupManip() { delete _popup; }
void PopupManip::Grasp (Event& e) { _popup->Popup(e); }
void PopupManip::Effect (Event&) { _popup->Unselect(); }
 
boolean PopupManip::Manipulating (Event& e) {
    e.target->Handle(e);
    return e.leftmouse;
}
 
void PopupManip::SetViewer (Viewer* viewer) { _viewer = viewer; }
void PopupManip::SetTool (Tool* tool) { _tool = tool; }
Viewer* PopupManip::GetViewer () { return _viewer; }
Tool* PopupManip::GetTool () { return _tool; }

//------------------------wire manipulator------------------------

WireManip::WireManip(Viewer* v, RubberWire* rv, Transformer* t, Tool* tool) :
					 VertexManip(v, rv, t, tool)
{
    _target = nil;
}
 
boolean WireManip::Manipulating (Event& e)
{
    Rubberband* r = GetRubberband();
 
    if (e.eventType == MotionEvent) {
        Constrain(e);
        r->Track(e.x, e.y);
 
    } else if (e.eventType == DownEvent) {
        Constrain(e);
        GetGrowingVertices()->AppendVertex(e.x, e.y);
 
        if (e.button == MIDDLEMOUSE) {
            return false;
        } else {
            Grasp(e);
        }
    }
    return true;
}
 
void WireManip::Constrain (Event& e)
{
    GraphicView* view = GetViewer()->GetGraphicView();
    _target =
view->ConnectorIntersecting(e.x-SLOP,e.y-SLOP,e.x+SLOP,e.y+SLOP);
 
    if (_target != nil) {
        float cx, cy;
        _target->GetGraphic()->GetCenter(cx, cy);
        e.x = round(cx);
        e.y = round(cy);
    }
    GetRubberWire()->Manhattan(!e.shift);
    if(e.shift)
    	GetRubberband()->Draw();
}
 
RubberWire* WireManip::GetRubberWire() {return (RubberWire*) GetRubberband();}

