/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include "dctclasses.h"
#include "dctvars.h"
#include "dcted.h"

#include <Unidraw/Components/component.h>

#include "stdio.h"
#include "stream.h"
#include "string.h"
#include "stdlib.h"

extern "C" {
#include "dbStaticLib.h"
};

// state variables for nodes and pv links

/*****************************Nodes*****************************************/

NodeVar::NodeVar(const char* name, char* f, char* r) : NameVar(name)
{
    if(f) _field=strdup(f);		else _field=nil;
    if(r) _record=strdup(r);	else _record=nil;
}

NodeVar::~NodeVar()
{
	if(_field) delete [] _field;
	if(_record) delete [] _record;
}

void NodeVar::SetRecField(char* f,char* r)
{
    if ( _field==nil || _record==nil || 
		strcmp(_field,f) || strcmp(_record,r) ||
		_field[0]=='\0' || _record[0]=='\0' )
	{
		if(_field) delete [] _field;
		if(_record) delete [] _record;
		if(f==NULL || f[0]=='\0') f="VAL";
        _field = strdup(f);
		_record = strdup(r);
        Notify();
    }
}

StateVar& NodeVar::operator=(StateVar& var)
{
	char* f;
    NameVar::operator=(var);

    if (var.IsA(NODE_VAR))
	{
        NodeVar* nodeVar = (NodeVar*) &var;
		// verify that GetField() actually has a value
		// default to VAL if GetField returns NULL
		f=nodeVar->GetField();
		if(f==NULL || f[0]=='\0') f="VAL";
        SetRecField(f,nodeVar->GetRecord());
		// cerr << " " << f << "\n";
    }
    return *this;
}

StateVar* NodeVar::Copy () { return new NodeVar(GetName(), _field, _record); }

void NodeVar::Read (istream& in)
{
	char f[5],r[50];

	// get the name of the field and record from in
    NameVar::Read(in);
	in >> r >> f;
	// cerr << "nodevar::read got (" << r << "." << f << ")\n";

	_field=strdup(f);
	_record=strdup(r);

	char junko[50]; // this is test garbage, the word "nodevar"
	in >> junko;
}

void NodeVar::Write (ostream& out)
{
	char* f=_field;

    NameVar::Write(out);
	if(_record) out << _record << " ";
	else out << "0 ";

	if(f==NULL || f[0]=='\0') f="VAL";

	out << f << " ";
	out << "nodevar ";
}

ClassId NodeVar::GetClassId () { return NODE_VAR; }
boolean NodeVar::IsA (ClassId id) { return NODE_VAR == id || NameVar::IsA(id);}

/*****************************PV Links**************************************/

PVLinkVar::PVLinkVar(Component* c, DBENTRY* r, const char* p,const char* m) :
	NameVar("pvlink")
{
    if(r)
	{
		dbCopyEntryContents(r,&_record);
		valid_record=1;
	}
	else
		valid_record=0;

	_comp=c;
	_pp=strdup(p);
	_ms=strdup(m);
}

PVLinkVar::~PVLinkVar()
{
	delete [] _pp;
	delete [] _ms;
}

void PVLinkVar::SetRecField(DBENTRY* r)
{
    if(r)
	{
		dbCopyEntryContents(r,&_record);
		valid_record=1;
	}
	else
		valid_record=0;

	Notify();
	_comp->Update();
}

void PVLinkVar::SetLink(const char* p,const char* m)
{
    if ( strcmp(_pp,p) || strcmp(_ms,m) )
	{
		delete [] _pp;
		delete [] _ms;
        _pp = strdup(p);
		_ms = strdup(m);
        Notify();
		_comp->Update();
    }
}

const char* PVLinkVar::GetLinkString()
{
	strcpy(_ppms,_pp);
	strcat(_ppms,"/");
	strcat(_ppms,_ms);
	return _ppms;
}

StateVar& PVLinkVar::operator = (StateVar& var)
{
	NameVar::operator=(var);
	PVLinkVar* linkVar;

    if (var.IsA(PVLINK_VAR))
	{
        linkVar=(PVLinkVar*)&var;
        SetRecField(linkVar->GetRecord());
        SetLink(linkVar->GetLinkPP(),linkVar->GetLinkMS());
    }
    return *this;
}

StateVar* PVLinkVar::Copy() { return new PVLinkVar(_comp,&_record); }

void PVLinkVar::Read (istream& in)
{
	char rec_name[50], field_name[50];
	DctUnidraw* du = (DctUnidraw*)unidraw;
	DBBASE* db = du->GetCatDatabase();
	char tmp_pp[5], tmp_ms[5], junko[50];

	if(db==NULL) db=du->GetDatabase();
    NameVar::Read(in);

	in >> field_name >> rec_name;
	in >> tmp_pp >> tmp_ms >> (char*)_ppms;

	_pp = strdup(tmp_pp);
	_ms = strdup(tmp_ms);

	// cerr << "pvlinkvar::read got (" << rec_name << "." << field_name ;
	// cerr << ")(" << _pp << ")(" << _ms << ")\n";

	dbInitEntry(db,&_record);

	if(dbFindRecord(&_record,rec_name))
	{
		// record does not exist
		cerr << "Graphics file not in sync with .db file\n";
		cerr << "   - .db file missing record " << rec_name << "\n";
		exit(1);
	}
	else if(dbFindField(&_record,field_name))
	{
		// field does not exist
		cerr << "Graphics file not in sync with .db file\n";
		cerr << "   - missing field " << rec_name << "." << field_name << "\n";
	}
	else
		valid_record=1;

	in >> junko;
}

void PVLinkVar::Write (ostream& out)
{
    NameVar::Write(out);
	if(valid_record)
      out << dbGetFieldName(&_record)<< " " << dbGetRecordName(&_record) << " ";
	else
      out << "JUNK JUNK ";

    out << _pp << " ";
	out << _ms << " ";

	if(_ppms[0]=='\0')
	{
		strcpy((char*)_ppms,_pp);
		strcat((char*)_ppms,"/");
		strcat((char*)_ppms,_ms);
	}

	out << _ppms << " ";
	out << "pvlinkvar ";
}

ClassId PVLinkVar::GetClassId () { return PVLINK_VAR; }
boolean PVLinkVar::IsA (ClassId id) {return PVLINK_VAR==id||NameVar::IsA(id);}

