/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#define MAINLINE

#include <unistd.h>

#include <database.h>
#include <cursor.h>
#include <dbcomp.h>
#include <recdesc.h>
#include <klist.h>

#ifndef MAXFILES
#define MAXFILES 500
#endif

#ifndef FIXED_EPICS_DCTSDR
#define FIXED_EPICS_DCTSDR "/net/phebos/epics/epics/share/epicsH/rec/default.dctsdr"
#endif

#ifndef LOCAL_DCTSDR
#define LOCAL_DCTSDR "./default.dctsdr"
#endif

void parse_command_line_args(int, char **, BOOL *, char **, char **, char **);
void display_container(Container *, int);
void traverse_list_of_recdes(Recdeslist *);
void delete_recs_with_substring(DctDatabase *, char *);

static BOOL db_only=FALSE;

main(int argc, char **argv)
{

DctDatabase Db;
BOOL verbose;
char *ascii_filenames[MAXFILES];
char *dbname;
char *sdr_filename;
int i;

    parse_command_line_args(argc, argv, &verbose, 
	ascii_filenames, &dbname, &sdr_filename);

    Db.read_sdr(sdr_filename);

    Db.read_ascii_change_filename(ascii_filenames[0], TRUE);

    for (i = 1; i < MAXFILES && ascii_filenames[i]; i ++)
	Db.read_ascii(ascii_filenames[i], FALSE);

    if (dbname)
	Db.rename(dbname);

    Db.save_ascii_stdout(verbose);

    if(db_only==FALSE) Db.makedb(sdr_filename);

    exit(0); /* needed for auto-builds of databases from makefiles */

} /* end main() */

void parse_command_line_args(int argc, char **argv, 
    BOOL *verbose, char **ascii_filenames, char **dbname, char **sdr_filename)
{

extern char *optarg; /* needed for getopt() */
extern int optind;   /* needed for getopt() */
int i;
int c;
BOOL input_error;

    /* defaults */
    *verbose = FALSE;
    *dbname = (char *) NULL;

    *sdr_filename = (char *) NULL;
    input_error = FALSE;

    while (!input_error && (c = getopt(argc, argv, "cvn:s:")) != -1)
    {
        switch (c)
        {
            case 'c': db_only = TRUE; break;
            case 'v': *verbose = TRUE; break;
            case 'n':
                if (*dbname)
		    /* already specified a dbname */
                    input_error = TRUE;
                else
		    *dbname = strdup(optarg);
                break;
            case 's':
                if (*sdr_filename)
		    /* already specified a sdr_filename */
                    input_error = TRUE;
                else
		    *sdr_filename = strdup(optarg);
                break;
            case '?': input_error = 1; break;
            default: input_error = 1; break;
        } /* end switch() */
    } /* endwhile */

    if (!input_error && optind < argc)
    {
	for (i = 0; i < MAXFILES; i ++)
	    ascii_filenames[i] = (char *) NULL;

	for (i = 0; i < MAXFILES && optind < argc; optind++, i ++)
	    ascii_filenames[i] = strdup(argv[optind]);

	if (i == MAXFILES)
	{
	    fprintf(stderr, "ERROR: too many files specified ... max %d\n",
		MAXFILES);
	    exit(1);
	} /* endif */
    }
    else
	input_error = TRUE;

    if (!input_error && !(*sdr_filename))
    {
	/* sdr_filename not specfied on command line */

	/* checking for local sdr_filename */
	if (!access(LOCAL_DCTSDR, R_OK))
	{
	    /* found local */
	    *sdr_filename = LOCAL_DCTSDR;
	}
	else
	{
	    /* local file not present */

	    /* checking for environment variable */
	    if (!(*sdr_filename = (char*) getenv("DCT_SDR_FILE")))
		/* did not find environment variable ... */
		/* using fixed epics sdrfile             */
		*sdr_filename = FIXED_EPICS_DCTSDR;
	} /* endif */
    } /* endif */

    if (input_error)
    {
        fprintf(stderr,
	"\nusage: %s [-v] [-c] [-n dbname] [-s sdr_fname] f1 [f2...fn]\n",argv[0]);

        fprintf(stderr,"\n\t-v\tVerbose mode, outputs all fields of\n");
	fprintf(stderr,"\t\trecords to standard out.  If not specified,\n");
	fprintf(stderr,"\t\tthen only fields found in the input file will\n");
	fprintf(stderr,"\t\tbe printed.\n\n");

	fprintf(stderr,"\t-c\tCreate db file only.  Concatenates f1-fn\n");
	fprintf(stderr,"\t\tand spits out a single db file to stdout.\n");
	fprintf(stderr,"\t\tNo .database file is created.  This option,\n");
	fprintf(stderr,"\t\talong with -v, could be used to get all\n");
	fprintf(stderr,"\t\tdefaults fields of a db file created with\n");
	fprintf(stderr,"\t\tgdct and vi.\n\n");

	fprintf(stderr,"\t-n\tNames the output database.  This will be\n");
	fprintf(stderr,"\t\tname of the .database file.  If not specified,\n");
	fprintf(stderr,"\t\tthe name of the database (and .database file)\n");
	fprintf(stderr,"\t\twill be taken from the name of the database\n");
	fprintf(stderr,"\t\tin file f1.\n\n");

	fprintf(stderr, "\t-s\tNames the default dctsdr file.  If not\n");
	fprintf(stderr, "\t\tspecified, it first checks for the local file\n");
	fprintf(stderr, "\t\t%s.  If that's not there, it then uses the\n",
	    LOCAL_DCTSDR);
	fprintf(stderr, "\t\tEPICS-provided file %s.\n\n", FIXED_EPICS_DCTSDR);

	fprintf(stderr,"\t\tAt least one input file must be specified.\n");

	fprintf(stderr,"\n\t\tRunning this program combines the records\n");
	fprintf(stderr,"\t\tin all the input files f1,f2,...,fn to \n");
	fprintf(stderr,"\t\tproduce two files: a .database file and\n");
	fprintf(stderr,"\t\tan ascii .db-type file that is written to\n");
	fprintf(stderr,"\t\tstandard out.\n\n");

	exit(1);
    } /* endif */


}

void delete_recs_with_substring(DctDatabase *dbp, char *cp)
{

RecDesc *rp;
Recdesnodecursor *rcp;

    if (dbp && cp)
    {
	/* deleting from nowhere() first */
	rcp = new Recdesnodecursor(dbp->get_recdeslist());

	for (rp = rcp->getfirst(); rp; rp = rcp->getnext())
	    rp->delete_recs_from_rlist_with_substring(cp);

	delete rcp;

	/* deleting from main container and on down */
	(dbp->get_universe())->delete_recs_from_container_with_substring(cp);

    }

} 

void traverse_list_of_recdes(Recdeslist *rlp)
{

RecDesc *rp;
Recdesnodecursor rc(rlp);

    for (rp = rc.getfirst(); rp; rp = rc.getnext())
    {
	printf("RecDesc %s\n", rp->name());
	rp->list_record_instances();
	printf("\n");
    } /* endfor */

}

void display_container(Container *cp, int lvl)
{

Dbcompnodecursor *curp;
Dbcomp *compp;
int i;

    if (cp)
    {
	if (cp->isa() == CONTAINER)
	{
	    for (i = 0; i < lvl; i ++) printf("   ");
	    printf("---Start container >%s<\n", cp->name());

	    curp = new Dbcompnodecursor(cp->get_list());

	    for (compp = curp->getfirst(); compp; compp = curp->getnext())
	    {
		if (compp->isa() == CONTAINER)
		    display_container((Container *) compp, lvl + 1);
		else
		{
		    for (i = 0; i < lvl+1; i ++) printf("   ");
		    printf("Record >%s<\n", ((Record *) compp)->name());
		} /* endif */
	    } /* endfor */

	    for (i = 0; i < lvl; i ++) printf("   ");
	    printf("---End container >%s<\n", cp->name());
	} /* endif */
    } /* endif */

}
